package io.github.null2264.tsukumogami.core.commands

import io.github.null2264.tsukumogami.core.Context
import kotlin.reflect.KFunction

interface IGroup {

    val allCommands: MutableMap<String, Command>

    private fun addCommand(command: Command) {
        allCommands[command.name] = command
    }

    fun commands(
        name: String,
        alias: Set<String> = setOf(),
        description: String = "",
        handler: suspend (Context) -> Unit,
    ) {
        val command = Command(name, alias, description, ::EmptyArguments) { ctx, _ -> handler(ctx) }
        addCommand(command)
    }

    fun <Args : Arguments> commands(
        name: String,
        alias: Set<String> = setOf(),
        description: String = "",
        arguments: KFunction<Args>,
        handler: suspend (Context, Args) -> Unit,
    ) {
        val command = Command(name, alias, description, arguments) { ctx, args -> handler(ctx, args as Args) }
        addCommand(command)
    }

    fun groups(
        name: String,
        alias: Set<String> = setOf(),
        description: String = "",
        declaration: IGroup.() -> Unit,
    ) {
        val group = Group(name, alias, description)
        declaration(group)
        addCommand(group)
    }
}
