package io.github.null2264.tsukumogami.core.commands.converters.impl

import dev.kord.common.entity.Snowflake
import dev.kord.core.entity.User
import io.github.null2264.tsukumogami.core.Context
import io.github.null2264.tsukumogami.core.annotation.TsukumogamiInternalApi
import io.github.null2264.tsukumogami.core.commands.converters.Converter
import io.github.null2264.tsukumogami.core.exceptions.CommandException
import io.github.null2264.tsukumogami.core.ext.users
import kotlinx.coroutines.flow.firstOrNull

class UserConverter : Converter<User>() {

    override lateinit var parsed: User

    override suspend fun parse(context: Context, input: String): User {
        if (input.equals("me", true)) {
            val user = context.author
            if (user != null) {
                this.parsed = user
                return this.parsed
            }
        }
        if (input.equals("you", true)) {
            this.parsed = context.bot.client.getSelf()
            return this.parsed
        }

        this.parsed = context.findUser(input) ?:
            throw CommandException("User not found")

        return this.parsed
    }

    private suspend fun Context.findUser(arg: String): User? =
        if (arg.startsWith("<@") && arg.endsWith(">")) {
            val id: String = arg.substring(2, arg.length - 1).replace("!", "")

            try {
                bot.client.getUser(Snowflake(id))
            } catch (_: NumberFormatException) {
                throw CommandException("Invalid user ID")
            }
        } else {
            try {
                bot.client.getUser(Snowflake(arg))
            } catch (_: NumberFormatException) {
                if (!arg.contains("#")) {
                    null
                } else {
                    bot.client.users.firstOrNull { user ->
                        user.tag.equals(arg, true)
                    }
                }
            }
        }
}
