package io.github.null2264.tsukumogami.core.ext

fun String.parseCommandAndArguments(): List<String> {
    if (isBlank()) {
        return emptyList()
    }

    val result = mutableListOf<String>()
    val currentToken = StringBuilder()
    var inQuotes = false

    for (char in this) {
        when (char) {
            '"' -> {
                if (inQuotes) {
                    // Closing quote: add the accumulated token
                    result.add(currentToken.toString())
                    currentToken.clear()
                    inQuotes = false
                } else {
                    // Opening quote:
                    // If there's an existing token (e.g., word"another"), add it first
                    if (currentToken.isNotEmpty()) {
                        result.add(currentToken.toString())
                        currentToken.clear()
                    }
                    inQuotes = true
                }
            }
            ' ' -> {
                if (inQuotes) {
                    // Space inside quotes: append it
                    currentToken.append(char)
                } else {
                    // Space outside quotes: separator
                    if (currentToken.isNotEmpty()) {
                        result.add(currentToken.toString())
                        currentToken.clear()
                    }
                    // Ignore multiple spaces between tokens
                }
            }
            else -> {
                // Any other character: append it
                currentToken.append(char)
            }
        }
    }

    // Add any remaining token after the loop (e.g., if the string doesn't end with a quote or space)
    if (currentToken.isNotEmpty()) {
        result.add(currentToken.toString())
    }

    return result
}
